@@ -1,9 +1,9 @@
 --
--- luatexja/ltj-jfmglue.lua
+-- ltj-jfmglue.lua
 --
 luatexbase.provides_module({
   name = 'luatexja.jfmglue',
-  date = '2018/08/11',
+  date = '2019/09/26',
   description = 'Insertion process of JFM glues, [x]kanjiskip and others',
 })
 luatexja.jfmglue = luatexja.jfmglue or {}
@@ -12,7 +12,8 @@ luatexja.load_module('base');      local ltjb = luatexja.base
 luatexja.load_module('stack');     local ltjs = luatexja.stack
 luatexja.load_module('jfont');     local ltjf = luatexja.jfont
 luatexja.load_module('direction'); local ltjd = luatexja.direction
-luatexja.load_module('setwidth');      local ltjw = luatexja.setwidth
+luatexja.load_module('setwidth');  local ltjw = luatexja.setwidth
+luatexja.load_module('lotf_aux');  local ltju = luatexja.lotf_aux
 local pairs = pairs
 
 --local to_node = node.direct.tonode
@@ -267,7 +268,7 @@ luatexbase.create_callback("luatexja.jfmglue.whatsit_after", "data",
 
 -- calc next Np
 local calc_np 
-do
+do -- 001 -----------------------------------------------
 
 local traverse = node.direct.traverse
 local function check_next_ickern(lp)
@@ -306,21 +307,38 @@ local function calc_np_pbox(lp, last)
    return lp
 end
 
-local ltjw_apply_ashift_math = ltjw.apply_ashift_math
-local ltjw_apply_ashift_disc = ltjw.apply_ashift_disc
-local min, max = math.min, math.max
-local function calc_np_aux_glyph_common(lp, acc_flag)
-   Np.nuc = lp
-   Np.first= (Np.first or lp)
-   if if_lang_ja(lp) then
+local calc_np_aux_glyph_common
+do -- 002 ---------------------------------------
+   local min, max = math.min, math.max
+   local getwhd = node.direct.getwhd
+   local attr_jchar_class = luatexbase.attributes['ltj@charclass']
+   local attr_jchar_code = luatexbase.attributes['ltj@charcode']
+   local font_getfont = font.getfont
+   local function calc_np_notdef(lp)
+      if not font_getfont(getfont(lp)).characters[getchar(lp)] then
+	 local ln = node_next(lp)
+	 if ltju.specified_feature(getfont(lp), 'notdef') and ln and getid(ln)==id_glyph then 
+	    set_attr(lp, attr_icflag, PROCESSED)
+	    set_attr(ln, attr_jchar_code, has_attr(lp, attr_jchar_code) or getchar(lp))
+	    set_attr(ln, attr_jchar_class, has_attr(lp, attr_jchar_class) or 0)
+	    Np.nuc, lp = ln, ln
+	 end
+      end
+      return lp
+   end 
+function calc_np_aux_glyph_common(lp, acc_flag)
+   Np.nuc, Np.first = lp, (Np.first or lp)
+   if if_lang_ja(lp) then -- JAchar
       Np.id = id_jglyph
       local m, mc, cls = set_np_xspc_jachar(Np, lp)
       local npi, npf
+      local w, h, d = getwhd(lp)
+      if w==0 and h==0 and d==0 then lp = calc_np_notdef(lp) end
       lp, head, npi, npf = capsule_glyph(lp, m, mc[cls], head, tex_dir)
       Np.first = (Np.first~=Np.nuc) and Np.first or npf or npi
       Np.nuc = npi
       return true, check_next_ickern(lp);
-   else
+   else --ALchar
       Np.id = id_glyph
       set_np_xspc_alchar(Np, getchar(lp), lp, 1)
       -- loop
@@ -403,8 +421,11 @@ local function calc_np_aux_glyph_common(lp, acc_flag)
       return true, lp
    end
 end
+end -- 002 ---------------------------------------
 local calc_np_auxtable
-do
+do  -- 002 ---------------------------------------
+local ltjw_apply_ashift_math = ltjw.apply_ashift_math
+local ltjw_apply_ashift_disc = ltjw.apply_ashift_disc
 local node_end_of_math = node.direct.end_of_math
 local dir_tate = luatexja.dir_table.dir_tate
 local sid_start_link = node.subtype('pdf_start_link')
@@ -445,6 +466,10 @@ calc_np_auxtable = {
 	    local lq = node_next(lp);
 	    head = node_remove(head, lp); node_free(lp); non_ihb_flag = false
 	    return false, lq;
+	 elseif getfield(lp, 'user_id')==luatexja.userid_table.JA_AL_BDD then
+	    local lq = node_next(lp);
+	    head = node_remove(head, lp); node_free(lp)
+	    return false, lq;
 	 else
 	    set_attr(lp, attr_icflag, PROCESSED)
 	    luatexbase.call_callback("luatexja.jfmglue.whatsit_getinfo",
@@ -513,7 +538,7 @@ calc_np_auxtable = {
       return false, node_next(lp)
    end,
 }
-end
+end -- 002 ---------------------------------------
 calc_np_auxtable[id_rule]   = calc_np_auxtable.box_like
 calc_np_auxtable[15]        = calc_np_auxtable.box_like
 
@@ -551,7 +576,7 @@ function calc_np(last, lp)
    end
    Np=nil
 end
-end
+end -- 001 -----------------------------------------------
 
 -- extract informations from Np
 -- We think that "Np is a Japanese character" if Np.met~=nil,
@@ -609,9 +634,11 @@ do
 	 local f = (lig ==1) and nullfunc or node_tail
          local xc, xs = getcomponents(x), getsubtype(x)
 	 while xc and xs and xs%4>=2 do
-	    x = f(xc); xc, xs = getcomponents(x), getsubtype(x)
+	    x = f(xc);
+	    if getid(x)==id_disc then x, xc, xs = nil, getfield(x,'replace'), 2
+	    else xc, xs = getcomponents(x), getsubtype(x) end
 	 end
-	 c = getchar(x)
+	 c = x and getchar(x) or c
 	 Nx.pre  = table_current_stack[PRE + c]  or 0
 	 Nx.post = table_current_stack[POST + c] or 0
       else
@@ -1253,7 +1280,7 @@ end
 end
 
 do
-   local IHB  = luatexja.userid_table.IHB
+   local IHB  = luatexja.userid_table.IHB 
    local BPAR = luatexja.userid_table.BPAR
    local BOXB = luatexja.userid_table.BOXB
    local node_prev = node.direct.getprev
