@@ -1,11 +1,12 @@
 --
--- src/ltj-setwidth.lua
+-- ltj-setwidth.lua
 --
 
 luatexja.load_module('base');      local ltjb = luatexja.base
 luatexja.load_module('stack');     local ltjs = luatexja.stack
 luatexja.load_module('jfont');     local ltjf = luatexja.jfont
 luatexja.load_module('direction'); local ltjd = luatexja.direction
+luatexja.load_module('lotf_aux');  local ltju = luatexja.lotf_aux
 
 local setfield = node.direct.setfield
 local getfield = node.direct.getfield
@@ -14,6 +15,7 @@ local getfont = node.direct.getfont
 local getlist = node.direct.getlist
 local getchar = node.direct.getchar
 local getsubtype = node.direct.getsubtype
+local getwidth = node.direct.getwidth or function(n) return getfield(n,'width') end
 local getdepth = node.direct.getdepth or function(n) return getfield(n,'depth') end
 local getwhd = node.direct.getwhd or function(n)
   return getfield(n,'width'), getfield(n,'height'),getfield(n,'depth') end
@@ -23,6 +25,7 @@ local setwhd = node.direct.setwhd or function(n,w,h,d)
 local setchar = node.direct.setchar or function(n,c) setfield(n,'char',c) end
 local setnext = node.direct.setnext or function(n,c) setfield(n,'next',c) end
 local setdir = node.direct.setdir or function(n,c) setfield(n,'dir',c) end
+local setkern = node.direct.setkern or function(n,c) setfield(n,'kern',c) end
 local setoffsets = node.direct.setoffsets or function(n,x,y)
   setfield(n,'xoffset',x); setfield(n,'yoffset',y)  end
 local getoffsets = node.direct.getoffsets or function(n)
@@ -56,6 +59,7 @@ local dir_tate = luatexja.dir_table.dir_tate
 local attr_ykblshift = luatexbase.attributes['ltj@ykblshift']
 local attr_tkblshift = luatexbase.attributes['ltj@tkblshift']
 local attr_icflag = luatexbase.attributes['ltj@icflag']
+local attr_vert_ori = luatexbase.attributes['ltj@vert@ori']
 
 local ltjf_font_extra_info = ltjf.font_extra_info
 
@@ -81,7 +85,7 @@ luatexbase.create_callback("luatexja.set_width", "data",
 			   end)
 local call_callback = luatexbase.call_callback
 
-local fshift =  { down = 0, left = 0}
+local fshift =  { down = 0, left = 0 }
 
 local min, max = math.min, math.max
 
@@ -170,22 +174,21 @@ local function capsule_glyph_tate_rot(p, met, char_data, head, dir, asc)
    return q, head, box
 end
 
+local font_getfont = font.getfont
+local get_ascender, get_descender = ltju.get_ascender, ltju.get_descender
 local function capsule_glyph_tate(p, met, char_data, head, dir)
    if not char_data then return node_next(p), head end
-   local ascent, descent = met.ascent, met.descent
-   local fwidth, pwidth = char_data.width
+   local fwidth, pwidth, ascender = char_data.width
    do
-      local pf = getfont(p)
-      local pc = getchar(p)
-      local cei = ltjf_font_extra_info[pf] and  ltjf_font_extra_info[pf][pc]
-      if cei and met.vert_activated then
-	 if cei.rotation then
-	    return capsule_glyph_tate_rot(p, met, char_data, head, dir, 0.5*(ascent-descent))
+      local pf, pc = getfont(p), getchar(p)
+      local feir = ltjf_font_extra_info[pf]
+      if feir and feir.rotation and met.vert_activated then
+	 if feir.rotation[pc] and (has_attr(p, attr_vert_ori) or 0)<=0 then
+	    return capsule_glyph_tate_rot(p, met, char_data, head, dir, 
+              0.5*(get_ascender(pf)-get_descender(pf)))
 	 end
       end
-      pwidth = (cei and cei.vwidth) and cei.vwidth * met.size or (ascent+descent)
-      pwidth = pwidth + (met.v_advance[pc] or 0)
-      ascent = met.v_origin[pc] and ascent - met.v_origin[pc] or ascent
+      pwidth, ascender = feir.vheight[pc]*met.size, feir.vorigin[pc]*met.size
    end
    fwidth = fwidth or pwidth
    fshift.down = char_data.down; fshift.left = char_data.left
@@ -202,13 +205,13 @@ local function capsule_glyph_tate(p, met, char_data, head, dir)
    setdir(box, dir)
 
    setoffsets(p, -fshift.down,
-              yo -(ascent + char_data.align*(fwidth-pwidth) - fshift.left) )
+              yo -(ascender + char_data.align*(fwidth-pwidth) - fshift.left) )
    local ws = node_new(id_whatsit, sid_save)
    local wm = node_new(id_whatsit, sid_matrix)
    setfield(wm, 'data', '0 1 -1 0')
-   local pwnh = -round(0.5*getfield(p, 'width'))
-   local k2 = node_new(id_kern, 1); setfield(k2, 'kern', pwnh)
-   local k3 = node_new(id_kern, 1); setfield(k3, 'kern', -getfield(p, 'width')-pwnh)
+   local pwnh = -round(0.5*getwidth(p))
+   local k2 = node_new(id_kern, 1); setkern(k2, pwnh)
+   local k3 = node_new(id_kern, 1); setkern(k3, -getwidth(p)-pwnh)
    local wr = node_new(id_whatsit, sid_restore)
    setfield(box, 'head', ws)
    setnext(ws, wm);  setnext(wm, k2);
@@ -224,7 +227,7 @@ luatexja.setwidth.capsule_glyph_tate = capsule_glyph_tate
 
 local function capsule_glyph_math(p, met, char_data)
    if not char_data then return nil end
-   local fwidth, pwidth = char_data.width, getfield(p, 'width')
+   local fwidth, pwidth = char_data.width, getwidth(p)
    fwidth = fwidth or pwidth
    fshift.down = char_data.down; fshift.left = char_data.left
    fshift = call_callback("luatexja.set_width", fshift, met, char_data)
